#
# SOMA - Copyright (C) CEA, 2015
# Distributed under the terms of the CeCILL-B license, as published by
# the CEA-CNRS-INRIA. Refer to the LICENSE file or to
# http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
# for details.
#

# Soma import
from .Str import StrControlWidget
from .Float import FloatControlWidget
from .Int import IntControlWidget
from .Enum import EnumControlWidget
from .List import ListControlWidget
from .Bool import BoolControlWidget
from .File import FileControlWidget
from .Directory import DirectoryControlWidget
from .Dict import DictControlWidget
from .Controller import ControllerControlWidget

# Define a structure that will contain the mapping between the string trait
# descriptions and the associated control classes
controls = {}

# Register all control class
controls["Str"] = StrControlWidget
controls["Unicode"] = StrControlWidget
controls["String"] = StrControlWidget
controls["Any"] = StrControlWidget
controls["Float"] = FloatControlWidget
controls["Int"] = IntControlWidget
controls["Enum"] = EnumControlWidget
controls["Bool"] = BoolControlWidget
controls["File"] = FileControlWidget
controls["Directory"] = DirectoryControlWidget
controls["List"] = ListControlWidget
controls["Instance"] = ControllerControlWidget
controls["TraitInstance"] = ControllerControlWidget
controls["ControllerTrait"] = ControllerControlWidget
controls["Dict"] = DictControlWidget



# -*- coding: utf-8 -*-
import json
import os
import collections


class StudyData():
    _instance = None
    """Class to write and save informations about process in the json"""

    def __init__(self):
        self.name_study = "Study"
        self.study_directory = ""
        self.compteur_run_process = {}
        self.runs = collections.OrderedDict()

    @staticmethod
    def get_instance():
        if StudyData._instance is None:
            StudyData._instance = StudyData()
            return StudyData._instance
        else:
            return StudyData._instance

    """Save on json with OrderedDict"""

    def save(self):
        self.dico = collections.OrderedDict(
            [('name_study', self.name_study), ('study_directory', self.study_directory), ('compteur_run_process', self.compteur_run_process), ('runs', self.runs)])
        json_string = json.dumps(self.dico, indent=4, separators=(',', ': '))
        with open(os.path.join(self.study_directory, self.name_study + '.json'), 'w') as f:
            f.write(unicode(json_string))

    """Load and put on self.__dict__ OrderedDict"""

    def load(self):
        try:
            with open(os.path.join(self.study_directory, self.name_study + '.json'), 'r') as json_data:
                self.__dict__ = json.load(
                    json_data, object_pairs_hook=collections.OrderedDict)
        # No file to load
        except IOError:
            pass

    """Get number of run process and iterate"""

    def inc_nb_run_process(self, process_name):
        if self.compteur_run_process.has_key(process_name):
            valeur = self.compteur_run_process[process_name]
            self.compteur_run_process[process_name] = valeur + 1
        else:
            self.compteur_run_process[process_name] = 1



#
# SOMA - Copyright (C) CEA, 2015
# Distributed under the terms of the CeCILL-B license, as published by
# the CEA-CNRS-INRIA. Refer to the LICENSE file or to
# http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
# for details.
#

# System import
import sys
import types
from textwrap import wrap
import re
import logging

# Define the logger
logger = logging.getLogger(__name__)

# Trait import
import traits.api as traits


# Global parameters
_type_to_trait_id = {
    int: "Int",
    unicode: "Unicode",
    str: "Str",
    float: "Float"
}
# In order to convert nipype special traits, we define a dict of
# correspondances
_trait_cvt_table = {
    "InputMultiPath_TraitCompound": "List",
    "InputMultiPath": "List",
    "MultiPath": "List",
    "Dict_Str_Str": "DictStrStr",
    "OutputMultiPath_TraitCompound": "List",
    "OutputMultiPath": "List",
    "OutputList": "List"
}


def get_trait_desc(trait_name, trait, def_val=None):
    """ Generate a trait string description of the form:

    [parameter name: type (default trait value) string help (description)]

    Parameters
    ----------
    name: string (mandatory)
        the trait name
    trait: a trait instance (mandatory)
        a trait instance
    def_val: object (optional)
        the trait default value
        If not in ['', None] add the default trait value to the trait
        string description.

    Returns
    -------
    manhelpstr: str
        the trait description.
    """
    # Get the trait description
    desc = trait.desc

    # Get the trait type
    trait_id = trait_ids(trait)

    # Add the trait name (bold)
    manhelpstr = ["{0}".format(trait_name)]

    # Get the default value string representation
    if def_val not in ["", None]:
        def_val = ", default value: {0}".format(repr(def_val))
    else:
        def_val = ""

    # Get the paramter type (optional or mandatory)
    if trait.optional:
        dtype = "optional"
    else:
        dtype = "mandatory"

    # Get the default parameter representation: trait type of default
    # value if specified
    line = "{0}".format(trait.info())
    if not trait.output:
        line += " ({0} - {1}{2})".format(trait_id, dtype, def_val)

    # Wrap the string properly
    manhelpstr = wrap(line, 70,
                      initial_indent=manhelpstr[0] + ": ",
                      subsequent_indent="    ")

    # Add the trait description if specified
    if desc:
        for line in desc.split("\n"):
            line = re.sub("\s+", " ", line)
            manhelpstr += wrap(line, 70,
                               initial_indent="    ",
                               subsequent_indent="    ")
    else:
        manhelpstr += wrap("No description.", 70,
                           initial_indent="    ",
                           subsequent_indent="    ")

    return manhelpstr


def is_trait_value_defined(value):
    """ Check if a trait value is valid.

    Parameters
    ----------
    value: type (mandatory)
        a value to test.

    Returns
    -------
    out: bool
        True if the value is valid,
        False otherwise.
    """
    # Initialize the default value
    is_valid = True

    # Check if the trait value is not valid
    if (value is None or value is traits.Undefined or
       (type(value) in types.StringTypes and value == "")):

        is_valid = False

    return is_valid


def is_trait_pathname(trait):
    """ Check if the trait is a file or a directory.

    Parameters
    ----------
    trait: CTrait (mandatory)
        the trait instance we want to test.

    Returns
    -------
    out: bool
        True if trait is a file or a directory,
        False otherwise.
    """
    return (isinstance(trait.trait_type, traits.File) or
            isinstance(trait.trait_type, traits.Directory))


def clone_trait(trait_description):
    """ Clone a trait from its string description.

    In the case of Enum trait, we need to initilaize the object with all its
    'trait_values'.

    Parameters
    ----------
    trait_description: list of str (mandatory)
        the trait string description from which we want to create a new trait
        instance of the same type.
    """
    # Build the new trait expression
    trait_expression = []

    # Go through all its possible types (Either trait structure)
    for trait_spec in trait_description:

        # Split the current trait specification
        trait_spec = trait_spec.split("_")
        trait_expression.append(build_expression_from_spec(trait_spec))

    # Build the final expression: use the Either trait if multiple expressions
    if len(trait_expression) > 1:
        expression = "eval_trait = traits.Either("
        for item in trait_expression:
            expression += "{0}, ".format(item)
        expression += ")"
    else:
        expression = "eval_trait = {0}".format(trait_expression[0])

    return eval_trait(expression)


def build_expression_from_spec(trait_spec):
    """ Build the expression to instanciate the trait.

    Parameters
    ----------
    trait_spec: list of string (mandatory)
        a trait string description structure.

    Returns
    -------
    expression: str
        the corresponding string expression.
    """
    # Only deal for now with those traits
    allowed_traits = ["List", "Tuple", "Int", "Float", "Str", "String", "File",
                      "Directory", "Any", "Bool"]

    # Build the expression we will evaluate to build the new trait item
    expression = ""
    expression_size = 0
    for trait_item in trait_spec:

        # Check item type
        if trait_item not in allowed_traits:
            raise ValueError("'{0}' trait not yet supported.".format(
                trait_item))

        # Update the expression with the new item
        expression += "traits.{0}(".format(trait_item)
        expression_size += 1

        # Tuple special case
        if trait_item == "Tuple":

            # Go through all tuple items
            for cnt, tuple_item in enumerate(trait_spec[expression_size:]):

                # Check item type
                if tuple_item not in allowed_traits:
                    raise ValueError("'{0}' trait not yet supported.".format(
                        trait_item))

                # List and Tuple special cases
                if tuple_item in ["List", "Tuple"]:
                    expression += build_expression_from_spec(
                        trait_spec[expression_size + cnt:])
                    break

                # Update expression with the new item
                else:
                    expression += "traits.{0}(), ".format(tuple_item)
            break

    # Close and store the current trait expression
    expression += ")" * expression_size

    return expression


def eval_trait(expression):
    """ Evaluate an expression to create a new trait.

    Parameters
    ----------
    expression: str (mandatory)
        a string expression to evaluate in order to create a new trait.

    Returns
    -------
    eval_trait: trait instance
        a trait instance.
    """
    # Create a new trait from its expression and namespace
    # Frist define the namespace were the expression will be executed
    namespace = {"traits": traits, "Undefined": traits.Undefined,
                 "eval_trait": None}

    # Complete the expression
    expression = "eval_trait = {0}".format(expression)

    # Debug message
    logger.debug("Evaluate expression '%s' in namespace '%s'", expression,
                 repr(namespace))

    # Evaluate the expression in the defined namespace
    def f():
        exec expression in namespace
    try:
        f()
    except:
        raise Exception(
            "Can't evaluate expression '{0}' in namespace '{1}'."
            "Please investigate: '{2}'.".format(
                expression, namespace, sys.exc_info()[1]))

    return namespace["eval_trait"]


def trait_ids(trait):
    """Return the type of the trait: File, Enum etc...

    Parameters
    ----------
    trait: trait instance (mandatory)
        a trait instance

    Returns
    -------
    main_id: list
        the string description (type) of the input trait.
    """
    # Get the trait class name
    handler = trait.handler or trait
    main_id = handler.__class__.__name__
    if main_id == "TraitCoerceType":
        real_id = _type_to_trait_id.get(handler.aType)
        if real_id:
            main_id = real_id

    # Use the convertion table to normalize the trait id
    if main_id in _trait_cvt_table:
        main_id = _trait_cvt_table[main_id]

    # Debug message
    logger.debug("Trait with main id %s", main_id)

    # Search for inner traits
    inner_ids = []

    # Either case
    if main_id in ["Either", "TraitCompound"]:

        # Debug message
        logger.debug("A coumpound trait has been found %s", repr(
            handler.handlers))

        # Build each trait compound description
        trait_description = []
        for sub_trait in handler.handlers:
            trait_description.extend(trait_ids(sub_trait()))
        return trait_description

    # Default case
    else:
        # FIXME may recurse indefinitely if the trait is recursive
        inner_id = '_'.join((trait_ids(i)[0]
                             for i in handler.inner_traits()))
        if not inner_id:
            klass = getattr(handler, 'klass', None)
            if klass is not None:
                inner_ids = [i.__name__ for i in klass.__mro__]
            else:
                inner_ids = []
        else:
            inner_ids = [inner_id]

        # Format the output string result
        if inner_ids:
            return [main_id + "_" + inner_desc for inner_desc in inner_ids]
        else:
            return [main_id]


def build_expression(trait):
    """ Build the expression to instanciate the trait.

    Parameters
    ----------
    trait: trait instance (mandatory)
        a trait instance.

    Returns
    -------
    expression: str
        the corresponding string expression.
    """
    # Get the trait desciption
    # If the desciption list return more than one element, we have to deal with
    # an Either trait
    trait_description = trait_ids(trait)

    # Error case
    if len(trait_description) == 0:
        raise ValueError("Can't deal with empty structure.")

    # Either case
    elif len(trait_description) > 1:

        # Debug message
        logger.debug("Either compounds are %s", repr(trait.handler.handlers))

        # Update expression
        either_expression = [build_expression(inner_trait())
                             for inner_trait in trait.handler.handlers]
        return "traits.Either({0})".format(", ".join(either_expression))

    # Default case
    else:
        trait_spec = trait_description[0].split("_")

    # Debug message
    logger.debug("Build expression from %s (%s)", repr(trait_spec),
                 repr(trait))

    # Standard case: add atomic trait description in the
    # expression
    trait_item = trait_spec[0]
    expression = "traits.{0}".format(trait_item)

    # Debug message
    logger.debug("Current item is a %s", trait_item)

    # Special case: Tuple
    # Need to set the value types
    if trait_item == "Tuple":

        # Debug message
        logger.debug("Inner traits are %s", repr(trait.get_validate()[1]))

        # Update expression
        tuple_expression = [build_expression(inner_trait())
                            for inner_trait in trait.get_validate()[1]]
        expression += "({0})".format(", ".join(tuple_expression))

    # Special case: List
    # Need to set the value type
    elif trait_item == "List":

        # Debug message
        logger.debug("Inner traits are %s", repr(trait.inner_traits))

        # Update expression
        expression += "({0})".format(build_expression(trait.inner_traits[0]))

    # Special case: Dict
    # Need to set the key and value types
    elif trait_item == "Dict":

        # Debug message
        logger.debug("Inner traits are %s", repr(trait.inner_traits))

        # Update expression
        expression += "({0}, {1})".format(
            build_expression(trait.inner_traits[0]),
            build_expression(trait.inner_traits[1]))

    # Special case: Enum
    # Need to add enum values at the construction
    elif trait_item == "Enum":

        # Debug message
        logger.debug("Default values are %s", repr(trait.get_validate()[1]))

        # Update expression
        expression += "({0})".format(trait.get_validate()[1])

    # Special case: Range
    # Need to add the lower and upper bounds
    elif trait_item == "Range":

        if isinstance(trait, traits.CTrait):
            # Debug message
            logger.debug("Range is %f - %f", trait.handler._low,
                         trait.handler._high)

            # Update expression
            expression += "(low={0},high={1})".format(
                trait.handler._low, trait.handler._high)
        else:
            # Debug message
            logger.debug("Range is %f - %f", trait._low, trait._high)

            # Update expression
            expression += "(low={0},high={1})".format(
                trait._low, trait._high)

    # Special case: File
    # Initialize the default file trait value to undefined
    elif trait_item == "File":
        expression += "(Undefined)"

    # Default
    else:
        expression += "()"

    return expression



#! /usr/bin/env python

from ez_setup import use_setuptools
use_setuptools()

import os
from setuptools import setup, find_packages
from datetime import datetime

python_dir = os.path.join(os.path.dirname(__file__), "python")
release_info = {}
execfile(os.path.join(python_dir, "soma", "info.py"), release_info)

setup(
    name=release_info["NAME"],
    description=release_info["DESCRIPTION"],
    long_description=release_info["LONG_DESCRIPTION"],
    license=release_info["LICENSE"],
    classifiers=release_info["CLASSIFIERS"],
    author=release_info["AUTHOR"],
    author_email=release_info["AUTHOR_EMAIL"],
    version=release_info["VERSION"],
    package_dir = {'': python_dir},
    packages=find_packages(python_dir),
    platforms=release_info["PLATFORMS"],
    #scripts=["../../bin/bv_minf_2_XML", ]
)



#
# Soma-base - Copyright (C) CEA, 2013
# Distributed under the terms of the CeCILL-B license, as published by
# the CEA-CNRS-INRIA. Refer to the LICENSE file or to
# http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html
# for details.
#

'''Compatibility module for PyQt and PySide. Currently supports PyQt4 and
PySide, not PyQt5.
This modules handles differences between PyQt and PySide APIs and behaviours,
and offers a few functions to make it easier to build neutral GUI code, which
can run using either backend.

The main funcion here is set_qt_backend() which must be called to initialize
the appropriate backend. Most functions of this module assume set_qt_backend()
has been called first to setup internal variables.

Note that such compatibility generally requires to use PyQt4 with SIP API
version 2, ie do not use QString, QVariant, QDate and similar classes, but
directly convert to/from python types, which is also PySide behaviour. The
qt_backend module switches to this API level 2, but this only works before the
PyQt modules are imported, thus it may fail if PyQt has already be imported
without such settings.

Qt submodules can be imported in two ways:

>>> from soma.qt_gui import qt_backend
>>> qt_backend.import_qt_submodule('QtWebKit')

or using the import statement:

>>> from soma.qt_gui.qt_backend import QtWebKit

in the latter case, set_qt_backend() will be called automatically to setup the
appropriate Qt backend, so that the use of the backend selection is more
transparent.
'''

import logging
import sys
import os
import imp
from soma.utils.functiontools import partial


# make qt_backend a fake module package, with Qt modules as sub-modules
__package__ = __name__
__path__ = [os.path.dirname(__file__)]

# internal variable to avoid warning several times
_sip_api_set = False

qt_backend = None


class QtImporter(object):

    def find_module(self, fullname, path=None):
        modsplit = fullname.split('.')
        modpath = '.'.join(modsplit[:-1])
        module_name = modsplit[-1]
        if modpath != __name__ or module_name == 'sip':
            return None
        set_qt_backend()
        qt_module = get_qt_module()
        found = imp.find_module(module_name, qt_module.__path__)
        return self

    def load_module(self, name):
        qt_backend = get_qt_backend()
        module_name = name.split('.')[-1]
        __import__('.'.join([qt_backend, module_name]))
        return sys.modules['.'.join([qt_backend, module_name])]

# tune the import statement to get Qt submodules in this one
sys.meta_path.append(QtImporter())


def get_qt_backend():
    '''get currently setup or loaded Qt backend name: "PyQt4" or "PySide"'''
    global qt_backend
    if qt_backend is None:
        pyside = sys.modules.get('PySide')
        if pyside is not None:
            qt_backend = 'PySide'
        else:
            pyqt = sys.modules.get('PyQt4')
            if pyqt is not None:
                qt_backend = 'PyQt4'
    return qt_backend


def set_qt_backend(backend=None, pyqt_api=1):
    '''set the Qt backend.

    If a different backend has already setup or loaded, a warning is issued.
    If no backend is specified, try to guess which one is already loaded.

    If no backend is loaded yet, try to behave like IPython does.
    See: https://ipython.org/ipython-doc/dev/interactive/reference.html#pyqt-and-pyside

    More precisely this means:
    * If QT_API environement variable is not set, use PyQt4, with PyQt API v1
    * if QT_API is set to "pyqt", use PyQt4, with PyQt API v2
    * if QT_API is set to "pyside", use PySide

    Moreover if using PyQt4, QtCore is patched to duplicate QtCore.pyqtSignal
    and QtCore.pyqtSlot as QtCore.Signal and QtCore.Slot. This is meant to ease
    code portability between both worlds.

    Parameters
    ----------
    backend: str (default: None)
        name of the backend to use
    pyqt_api: int (default: 1)
        PyQt API version: 1 or 2, only useful for PyQt4

    Examples
    --------
        >>> from soma.qt_gui import qt_backend
        >>> qt_backend.set_qt_backend('PySide')
        >>> qt_backend.import_qt_submodule('QtCore')
        <module 'PySide.QtCore' from '/usr/lib/python2.7/dist-packages/PySide/QtCore.so'>
    '''
    global qt_backend
    get_qt_backend()
    if backend is None:
        if qt_backend is None:
            # try to get from the environment variable QT_API, complying to
            # ETS 4
            # see
            # https://ipython.org/ipython-doc/dev/interactive/reference.html#pyqt-and-pyside
            qt_api = os.getenv('QT_API')
            if qt_api == 'pyqt':
                backend = 'PyQt4'
                pyqt_api = 2
            elif qt_api == 'pyside':
                backend = 'PySide'
            else:
                backend = 'PyQt4'
                pyqt_api = 1
        else:
            backend = qt_backend
    if qt_backend is not None and qt_backend != backend:
        logging.warn('set_qt_backend: a different backend, %s, has already '
                     'be set, and %s is now requested' % (qt_backend, backend))
    if backend == 'PyQt4':  # and sys.modules.get('PyQt4') is None:
        import sip
        if pyqt_api == 2:
            sip_classes = ['QString', 'QVariant', 'QDate', 'QDateTime',
                           'QTextStream', 'QTime', 'QUrl']
            global _sip_api_set
            for sip_class in sip_classes:
                try:
                    sip.setapi(sip_class, pyqt_api)
                except ValueError, e:
                    if not _sip_api_set:
                        logging.warning(e.message)
            _sip_api_set = True
    qt_module = __import__(backend)
    __import__(backend + '.QtCore')
    __import__(backend + '.QtGui')
    qt_backend = backend
    if backend == 'PyQt4':
        qt_module.QtCore.Signal = qt_module.QtCore.pyqtSignal
        qt_module.QtCore.Slot = qt_module.QtCore.pyqtSlot


def get_qt_module():
    '''Get the main Qt module (PyQt4 or PySide)'''
    global qt_backend
    return sys.modules.get(qt_backend)


def import_qt_submodule(submodule):
    '''Import a specified Qt submodule.
    An alternative to the standard statement:

    >>> from soma.qt_gui.qt_backend import <submodule>

    The main differences is that it forces loading the module from the
    appropriate backend, whereas the import statement will reuse the already
    loaded one. Moreover it returns the module.

    For instance,

    >>> from soma.qt_gui import qt_backend
    >>> qt_backend.set_qt_backend('PyQt4')
    >>> from soma.qt_gui.qt_backend import QtWebKit
    >>> QtWebKit
    <module 'PyQt4.QtWebKit' from '/usr/lib/python2.7/dist-packages/PyQt4/QtWebKit.so'>
    >>> qt_backend.set_qt_backend('PySide') # changing backend
    WARNING:root:set_qt_backend: a different backend, PyQt4, has already be set, and PySide is now requested
    >>> from soma.qt_gui.qt_backend import QtWebKit
    >>> QtWebKit
    <module 'PyQt4.QtWebKit' from '/usr/lib/python2.7/dist-packages/PyQt4/QtWebKit.so'>

    In the above example, we are still using the QtWebKit from PyQt4.
    Now:

    >>> QtWebKit = qt_backend.import_qt_submodule('QtWebKit')
    >>> QtWebKit
    <module 'PySide.QtWebKit' from '/usr/lib/python2.7/dist-packages/PySide/QtWebKit.so'>

    We are now actually using PySide.
    Note that it is generally a bad idea to mix both...

    Parameters
    ----------
        submodule: str (mandatory)
            submodule name, ex: QtWebKit

    Returns
    -------
        the loaded submodule
    '''
    __import__(qt_backend + '.' + submodule)
    mod = sys.modules[qt_backend + '.' + submodule]
    return mod


def _iconset(self, prop):
    return QtGui.QIcon(os.path.join(self._basedirectory,
                                    prop.text).replace("\\", "\\\\"))


def _pixmap(self, prop):
    return QtGui.QPixmap(os.path.join(self._basedirectory,
                                      prop.text).replace("\\", "\\\\"))


def loadUi(ui_file, *args, **kwargs):
    '''Load a .ui file and returns the widget instance.

    This function is a replacement of PyQt4.uic.loadUi. The only difference is
    that relative icon or pixmap file names that are stored in the *.ui file
    are considered to be relative to the directory containing the ui file. With
    PyQt4.uic.loadUi, relative file names are considered relative to the
    current working directory therefore if this directory is not the one
    containing the ui file, icons cannot be loaded.
    '''
    if get_qt_backend() == 'PyQt4':
        # the problem is corrected in version > 4.7.2,
        from PyQt4 import QtCore
        if QtCore.PYQT_VERSION > 0x040702:
            from PyQt4 import uic
            return uic.loadUi(ui_file, *args, **kwargs)
        else:
            # needed import and def
            from PyQt4.uic.Loader import loader
            if not hasattr(globals(), 'partial'):
                from soma.functiontools import partial

            def _iconset(self, prop):
                return QtGui.QIcon(os.path.join(self._basedirectory, prop.text).replace("\\", "\\\\"))

            def _pixmap(self, prop):
                return QtGui.QPixmap(os.path.join(self._basedirectory, prop.text).replace("\\", "\\\\"))
            uiLoader = loader.DynamicUILoader()
            uiLoader.wprops._basedirectory = os.path.dirname(
                os.path.abspath(ui_file))
            uiLoader.wprops._iconset = partial(_iconset, uiLoader.wprops)
            uiLoader.wprops._pixmap = partial(_pixmap, uiLoader.wprops)
            return uiLoader.loadUi(ui_file, *args, **kwargs)
    else:
        from PySide.QtUiTools import QUiLoader
        return QUiLoader().load(ui_file)  # , *args, **kwargs )


def loadUiType(uifile, from_imports=False):
    '''PyQt4 / PySide abstraction to uic.loadUiType.
    Not implemented for PySide, actually, because PySide does not have this
    feature.
    '''
    if get_qt_backend() == 'PyQt4':
        # the parameter from_imports doesn't exist in our version of PyQt
        from PyQt4 import uic
        return uic.loadUiType(uifile)
    else:
        raise NotImplementedError('loadUiType does not work with PySide')
        # ui = loadUi(uifile)
        # return ui.__class__, QtGui.QWidget # FIXME


def getOpenFileName(parent=None, caption='', directory='', filter='',
                    selectedFilter=None, options=0):
    '''PyQt4 / PySide compatible call to QFileDialog.getOpenFileName'''
    if get_qt_backend() == 'PyQt4':
        kwargs = {}
        # kwargs are used because passing None or '' as selectedFilter
        # does not work, at least in PyQt 4.10
        # On the other side I don't know if this kwargs works with older
        # sip/PyQt versions.
        if selectedFilter:
            kwargs['selectedFilter'] = selectedFilter
        if options:
            kwargs['options'] = QtGui.QFileDialog.Options(options)
        return get_qt_module().QtGui.QFileDialog.getOpenFileName(
            parent, caption, directory, filter, **kwargs)
    else:
        return get_qt_module().QtGui.QFileDialog.getOpenFileName(
            parent, caption, directory, filter, selectedFilter,
            QtGui.QFileDialog.Options(options))[0]


def getSaveFileName(parent=None, caption='', directory='', filter='',
                    selectedFilter=None, options=0):
    '''PyQt4 / PySide compatible call to QFileDialog.getSaveFileName'''
    if get_qt_backend() == 'PyQt4':
        kwargs = {}
        # kwargs are used because passing None or '' as selectedFilter
        # does not work, at least in PyQt 4.10
        # On the other side I don't know if this kwargs works with older
        # sip/PyQt versions.
        if selectedFilter:
            kwargs['selectedFilter'] = selectedFilter
        if options:
            kwargs['options'] = QtGui.QFileDialog.Options(options)
        return get_qt_module().QtGui.QFileDialog.getSaveFileName(parent,
                                                                 caption, directory, filter, **kwargs)
    else:
        return get_qt_module().QtGui.QFileDialog.getSaveFileName(parent,
                                                                 caption, directory, filter, selectedFilter, options)[0]


def getExistingDirectory(parent=None, caption='', directory='', options=None):
    '''PyQt4 / PySide compatible call to QFileDialog.getExistingDirectory'''
    if get_qt_backend() == 'PyQt4':
        kwargs = {}
        if options is not None:
            kwargs['options'] = QtGui.QFileDialog.Options(options)
        return get_qt_module().QtGui.QFileDialog.getExistingDirectory(parent,
                                                                      caption, directory, **kwargs)
    else:
        if options is not None:
            return get_qt_module().QtGui.QFileDialog.getExistingDirectory(
                parent, caption, directory,
                QtGui.QFileDialog.Options(options))[0]
        else:
            return get_qt_module().QtGui.QFileDialog.getExistingDirectory(
                parent, caption, directory)[0]


def init_matplotlib_backend():
    '''Initialize Matplotlib to use Qt, and the appropriate Qt/Python binding
    (PySide or PyQt) according to the configured/loaded toolkit.
    Moreover, the appropriate FigureCanvas type is set in the current module,
    and returned by this function.
    '''
    try:
        import matplotlib
    except ImportError:
        # if matplotlib cannot be found, don't do anything.
        return

    mpl_ver = [int(x) for x in matplotlib.__version__.split('.')[:2]]
    guiBackend = 'Qt4Agg'
    if 'matplotlib.backends' not in sys.modules:
        matplotlib.use(guiBackend)
    elif matplotlib.get_backend() != guiBackend:
        raise RuntimeError(
            'Mismatch between Qt version and matplotlib backend: '
            'matplotlib uses ' + matplotlib.get_backend() + ' but '
            + guiBackend + ' is required.')
    if get_qt_backend() == 'PySide':
        if 'backend.qt4' in matplotlib.rcParams.keys():
            matplotlib.rcParams['backend.qt4'] = 'PySide'
        else:
            raise RuntimeError("Could not use Matplotlib, the backend using "
                               "PySide is missing.")
    else:
        if 'backend.qt4' in matplotlib.rcParams.keys():
            matplotlib.rcParams['backend.qt4'] = 'PyQt4'
        else:
            # older versions of matplotlib used only PyQt4.
            if mpl_ver >= [1, 1]:
                raise RuntimeError("Could not use Matplotlib, the backend "
                                   "using PyQt4 is missing.")
    from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg \
        as FigureCanvas
    sys.modules[__name__].FigureCanvas = FigureCanvas
    return FigureCanvas
